@@ -23,7 +23,11 @@
 #
 #######################################################################
 
-package Argonaut::ClientDaemon::ModulesSystemd::Service;
+## no critic
+
+package Argonaut::ClientDaemon::Modules::Service;
+
+## use critic
 
 use strict;
 use warnings;
