@@ -73,7 +73,6 @@ BEGIN
       &argonaut_get_client_settings
       &argonaut_get_server_settings
       &argonaut_get_crawler_settings
-      &argonaut_get_ldap2zone_settings
       &argonaut_get_fuse_settings
     )],
     'file' => [qw(
@@ -606,6 +605,16 @@ sub argonaut_ldap_fsearch {
   return( $mesg, ${search_base} );
 }
 
+=item trim
+trims whitespaces from a given string
+=cut
+sub trim {
+  my $string = shift;
+  $string =~ s/^\s+//;
+  $string =~ s/\s+$//;
+  return $string;
+}
+
 #------------------------------------------------------------------------------
 # function for reading argonaut config
 #
@@ -613,12 +622,12 @@ sub argonaut_read_config {
   my %res = ();
   my $config = Config::IniFiles->new( -file => $configfile, -allowempty => 1, -nocase => 1);
 
-  $res{'server_ip'}       = $config->val( server  => "server_ip", "");
-  $res{'client_ip'}       = $config->val( client  => "client_ip", "");
-  $res{'ldap_configfile'} = $config->val( ldap    => "config",    "/etc/ldap/ldap.conf");
-  $res{'ldap_dn'}         = $config->val( ldap    => "dn",        "");
-  $res{'ldap_password'}   = $config->val( ldap    => "password",  "");
-  $res{'ldap_tls'}        = $config->val( ldap    => "tls",       "off");
+  $res{'server_ip'}       = trim($config->val( server  => "server_ip", ""));
+  $res{'client_ip'}       = trim($config->val( client  => "client_ip", ""));
+  $res{'ldap_configfile'} = trim($config->val( ldap    => "config",    "/etc/ldap/ldap.conf"));
+  $res{'ldap_dn'}         = trim($config->val( ldap    => "dn",        ""));
+  $res{'ldap_password'}   = trim($config->val( ldap    => "password",  ""));
+  $res{'ldap_tls'}        = trim($config->val( ldap    => "tls",       "off"));
 
   if ($res{'ldap_tls'} !~ m/^off|on$/i) {
     warn "Unknown value for option ldap/tls: ".$res{'ldap_tls'}." (valid values are on/off)\n";
@@ -644,7 +653,7 @@ sub argonaut_read_ldap_config {
       } else {
         if (($mesg->entries)[0]->get_value("$value")) {
           $config->{"$key"} = ($mesg->entries)[0]->get_value("$value");
-        } else {
+        } elsif (not defined $config->{"$key"}) {
           $config->{"$key"} = "";
         }
       }
@@ -685,7 +694,9 @@ sub argonaut_get_generic_settings {
     $settings->{'dn'}   = ($mesg->entries)[0]->dn();
     $settings->{'mac'}  = ($mesg->entries)[0]->get_value("macAddress");
     $settings->{'ip'}   = ($mesg->entries)[0]->get_value("ipHostNumber");
-    if (($mesg->entries)[0]->exists('gotoMode')) {
+    if (($mesg->entries)[0]->exists('fdMode')) {
+      $settings->{'locked'} = ($mesg->entries)[0]->get_value("fdMode") eq 'locked';
+    } elsif (($mesg->entries)[0]->exists('gotoMode')) {
       $settings->{'locked'} = ($mesg->entries)[0]->get_value("gotoMode") eq 'locked';
     } else {
       $settings->{'locked'} = 0;
@@ -709,7 +720,7 @@ sub argonaut_get_generic_settings {
     $mesg = $ldap->search( # perform a search
               base   => $ldap_base,
               filter => $filter,
-              attrs => [ 'dn', 'macAddress', 'gotoMode' ]
+              attrs => [ 'dn', 'macAddress', 'gotoMode', 'fdMode' ]
               );
     if (scalar($mesg->entries)>1) {
       die "Several computers matches $filter.$die_endl";
@@ -719,7 +730,9 @@ sub argonaut_get_generic_settings {
     $settings->{'dn'}   = ($mesg->entries)[0]->dn();
     $settings->{'mac'}  = ($mesg->entries)[0]->get_value("macAddress");
     $settings->{'ip'}   = ($mesg->entries)[0]->get_value("ipHostNumber");
-    if (($mesg->entries)[0]->exists('gotoMode')) {
+    if (($mesg->entries)[0]->exists('fdMode')) {
+      $settings->{'locked'} = ($mesg->entries)[0]->get_value("fdMode") eq 'locked';
+    } elsif (($mesg->entries)[0]->exists('gotoMode')) {
       $settings->{'locked'} = ($mesg->entries)[0]->get_value("gotoMode") eq 'locked';
     } else {
       $settings->{'locked'} = 0;
@@ -811,26 +824,6 @@ sub argonaut_get_crawler_settings {
   );
 }
 
-#------------------------------------------------------------------------------
-# get ldap2zone settings
-#
-sub argonaut_get_ldap2zone_settings {
-  return argonaut_get_generic_settings(
-    'argonautDNSConfig',
-    {
-      'binddir'       => 'argonautLdap2zoneBindDir',
-      'bindcachedir'  => 'argonautLdap2zoneBindCacheDir',
-      'allownotify'   => 'argonautLdap2zoneAllowNotify',
-      'allowupdate'   => 'argonautLdap2zoneAllowUpdate',
-      'allowtransfer' => 'argonautLdap2zoneAllowTransfer',
-      'ttl'           => 'argonautLdap2zoneTTL',
-      'rndc'          => 'argonautLdap2zoneRndc',
-      'noreverse'     => 'argonautLdap2zoneNoReverse',
-    },
-    @_
-  );
-}
-
 #------------------------------------------------------------------------------
 # get fuse settings
 #
